@@ -1,13 +1,15 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 import sqlalchemy
+import warnings
+
 from sqlalchemy import *
 
-from migrate import changeset
+from migrate import changeset, exceptions
 from migrate.changeset import *
 from migrate.changeset.schema import ColumnDelta
 from migrate.tests import fixture
-
+from migrate.tests.fixture.warnings import catch_warnings
 
 class TestAddDropColumn(fixture.DB):
     """Test add/drop column through all possible interfaces
@@ -164,23 +166,19 @@ class TestAddDropColumn(fixture.DB):
 
         # create column with fk
         col = Column('data', Integer, ForeignKey(reftable.c.id))
-        if self.url.startswith('sqlite'):
-            self.assertRaises(changeset.exceptions.NotSupportedError,
-                col.create, self.table)
+        col.create(self.table)
+
+        # check if constraint is added
+        for cons in self.table.constraints:
+            if isinstance(cons, sqlalchemy.schema.ForeignKeyConstraint):
+                break
         else:
-            col.create(self.table)
+            self.fail('No constraint found')
 
-            # check if constraint is added
-            for cons in self.table.constraints:
-                if isinstance(cons, sqlalchemy.schema.ForeignKeyConstraint):
-                    break
-            else:
-                self.fail('No constraint found')
+        # TODO: test on db level if constraints work
 
-            # TODO: test on db level if constraints work
-
-            self.assertEqual(reftable.c.id.name, col.foreign_keys[0].column.name)
-            col.drop(self.table)
+        self.assertEqual(reftable.c.id.name, col.foreign_keys[0].column.name)
+        col.drop(self.table)
 
         if self.engine.has_table(reftable.name):
             reftable.drop()
@@ -189,7 +187,7 @@ class TestAddDropColumn(fixture.DB):
     def test_pk(self):
         """Can create columns with primary key"""
         col = Column('data', Integer, nullable=False)
-        self.assertRaises(changeset.exceptions.InvalidConstraintError,
+        self.assertRaises(exceptions.InvalidConstraintError,
             col.create, self.table, primary_key_name=True)
         col.create(self.table, primary_key_name='data_pkey')
 
@@ -225,11 +223,11 @@ class TestAddDropColumn(fixture.DB):
 
         col.drop()
 
-    @fixture.usedb(not_supported='sqlite')
-    def test_unique(self):
-        """Can create columns with unique constraint"""
-        self.assertRaises(changeset.exceptions.InvalidConstraintError,
+    @fixture.usedb()
+    def test_unique_constraint(self):
+        self.assertRaises(exceptions.InvalidConstraintError,
             Column('data', Integer, unique=True).create, self.table)
+
         col = Column('data', Integer)
         col.create(self.table, unique_name='data_unique')
 
@@ -246,14 +244,33 @@ class TestAddDropColumn(fixture.DB):
         col.drop(self.table)
 
 # TODO: remove already attached columns with indexes, uniques, pks, fks ..
+
+    def _check_index(self,expected):
+        if 'mysql' in self.engine.name or 'postgres' in self.engine.name:
+            for index in tuple(
+                Table(self.table.name, MetaData(),
+                      autoload=True, autoload_with=self.engine).indexes
+                ):
+                if index.name=='ix_data':
+                    break
+            self.assertEqual(expected,index.unique)
+        
     @fixture.usedb()
     def test_index(self):
-        """Can create columns with indexes"""
-        self.assertRaises(changeset.exceptions.InvalidConstraintError,
-            Column('data', Integer).create, self.table, index_name=True)
         col = Column('data', Integer)
         col.create(self.table, index_name='ix_data')
 
+        self._check_index(False)
+
+        col.drop()
+        
+    @fixture.usedb()
+    def test_index_unique(self):
+        # shows how to create a unique index
+        col = Column('data', Integer)
+        col.create(self.table)
+        Index('ix_data', col, unique=True).create(bind=self.engine)
+
         # check if index was added
         self.table.insert(values={'data': 5}).execute()
         try:
@@ -264,9 +281,10 @@ class TestAddDropColumn(fixture.DB):
         else:
             self.fail()
 
-        Index('ix_data', col).drop(bind=self.engine)
-        col.drop()
+        self._check_index(True)
 
+        col.drop()
+        
     @fixture.usedb()
     def test_server_defaults(self):
         """Can create columns with server_default values"""
@@ -297,7 +315,132 @@ class TestAddDropColumn(fixture.DB):
     # TODO: test quoting
     # TODO: test non-autoname constraints
 
+    @fixture.usedb()
+    def test_drop_doesnt_delete_other_indexes(self):
+        # add two indexed columns
+        self.table.drop()
+        self.meta.clear()
+        self.table = Table(
+            self.table_name, self.meta,
+            Column('id', Integer, primary_key=True),
+            Column('d1', String(10), index=True),
+            Column('d2', String(10), index=True),
+            )
+        self.table.create()
 
+        # paranoid check
+        if SQLA_06:
+            self.refresh_table()
+            self.assertEqual(
+                sorted([i.name for i in self.table.indexes]),
+                [u'ix_tmp_adddropcol_d1', u'ix_tmp_adddropcol_d2']
+                )
+            
+        # delete one
+        self.table.c.d2.drop()
+
+        # ensure the other index is still there
+        if SQLA_06:
+            self.refresh_table()
+            self.assertEqual(
+                sorted([i.name for i in self.table.indexes]),
+                [u'ix_tmp_adddropcol_d1']
+                )
+        else:
+            # a crude test for 0.5.x
+            Index('ix_tmp_adddropcol_d1',self.table.c.d1).drop()
+            
+    def _actual_foreign_keys(self):
+        from sqlalchemy.schema import ForeignKeyConstraint
+        result = []
+        for cons in self.table.constraints:
+            if isinstance(cons,ForeignKeyConstraint):
+                col_names = []
+                for col_name in cons.columns:
+                    if not isinstance(col_name,basestring):
+                        col_name = col_name.name
+                    col_names.append(col_name)
+                result.append(col_names)
+        result.sort()
+        return result
+        
+    @fixture.usedb()
+    def test_drop_with_foreign_keys(self):
+        self.table.drop()
+        self.meta.clear()
+        
+        # create FK's target
+        reftable = Table('tmp_ref', self.meta,
+            Column('id', Integer, primary_key=True),
+        )
+        if self.engine.has_table(reftable.name):
+            reftable.drop()
+        reftable.create()
+        
+        # add a table with two foreign key columns
+        self.table = Table(
+            self.table_name, self.meta,
+            Column('id', Integer, primary_key=True),
+            Column('r1', Integer, ForeignKey('tmp_ref.id')),
+            Column('r2', Integer, ForeignKey('tmp_ref.id')),
+            )
+        self.table.create()
+
+        # paranoid check
+        self.assertEqual([['r1'],['r2']],
+                         self._actual_foreign_keys())
+        
+        # delete one
+        self.table.c.r2.drop()
+        
+        # check remaining foreign key is there
+        self.assertEqual([['r1']],
+                         self._actual_foreign_keys())
+        
+    @fixture.usedb()
+    def test_drop_with_complex_foreign_keys(self):
+        from sqlalchemy.schema import ForeignKeyConstraint
+        from sqlalchemy.schema import UniqueConstraint
+        
+        self.table.drop()
+        self.meta.clear()
+        
+        # create FK's target
+        reftable = Table('tmp_ref', self.meta,
+            Column('id', Integer, primary_key=True),
+            Column('jd', Integer),
+            UniqueConstraint('id','jd')
+            )
+        if self.engine.has_table(reftable.name):
+            reftable.drop()
+        reftable.create()
+        
+        # add a table with a complex foreign key constraint
+        self.table = Table(
+            self.table_name, self.meta,
+            Column('id', Integer, primary_key=True),
+            Column('r1', Integer),
+            Column('r2', Integer),
+            ForeignKeyConstraint(['r1','r2'],
+                                 [reftable.c.id,reftable.c.jd],
+                                 name='test_fk')
+            )
+        self.table.create()
+
+        # paranoid check
+        self.assertEqual([['r1','r2']],
+                         self._actual_foreign_keys())
+        
+        # delete one
+        self.table.c.r2.drop()
+        
+        # check the constraint is gone, since part of it
+        # is no longer there - if people hit this,
+        # they may be confused, maybe we should raise an error
+        # and insist that the constraint is deleted first, separately?
+        self.assertEqual([],
+                         self._actual_foreign_keys())
+        
 class TestRename(fixture.DB):
     """Tests for table and index rename methods"""
     level = fixture.DB.CONNECT
@@ -372,7 +515,7 @@ class TestRename(fixture.DB):
     
             # Index renames
             if self.url.startswith('sqlite') or self.url.startswith('mysql'):
-                self.assertRaises(changeset.exceptions.NotSupportedError,
+                self.assertRaises(exceptions.NotSupportedError,
                     self.index.rename, index_name2)
             else:
                 assert_index_name(index_name1)
@@ -448,16 +591,19 @@ class TestColumnChange(fixture.DB):
         self.assertEquals(num_rows(self.table.c.data, content), 1)
 
         # ...as a function, given a new object
-        col = Column('atad', String(40), server_default=self.table.c.data.server_default)
-        alter_column(self.table.c.data, col)
+        alter_column(self.table.c.data,
+                     name = 'atad', type=String(40),
+                     server_default=self.table.c.data.server_default)
         self.refresh_table(self.table.name)
         self.assert_('data' not in self.table.c.keys())
         self.table.c.atad   # Should not raise exception
         self.assertEquals(num_rows(self.table.c.atad, content), 1)
 
         # ...as a method, given a new object
-        col = Column('data', String(40), server_default=self.table.c.atad.server_default)
-        self.table.c.atad.alter(col)
+        self.table.c.atad.alter(
+            name='data',type=String(40),
+            server_default=self.table.c.atad.server_default
+            )
         self.refresh_table(self.table.name)
         self.assert_('atad' not in self.table.c.keys())
         self.table.c.data   # Should not raise exception
@@ -465,12 +611,7 @@ class TestColumnChange(fixture.DB):
         
     @fixture.usedb()
     def test_type(self):
-        """Can change a column's type"""
-        # Entire column definition given
-        self.table.c.data.alter(Column('data', String(42)))
-        self.refresh_table(self.table.name)
-        self.assert_(isinstance(self.table.c.data.type, String))
-        self.assertEquals(self.table.c.data.type.length, 42)
+        # Test we can change a column's type
 
         # Just the new type
         self.table.c.data.alter(type=String(43))
@@ -509,7 +650,7 @@ class TestColumnChange(fixture.DB):
 
         # Column object
         default = 'your_default'
-        self.table.c.data.alter(Column('data', String(40), server_default=DefaultClause(default)))
+        self.table.c.data.alter(type=String(40), server_default=DefaultClause(default))
         self.refresh_table(self.table.name)
         self.assert_(default in str(self.table.c.data.server_default.arg))
 
@@ -532,8 +673,8 @@ class TestColumnChange(fixture.DB):
         """Can change a column's null constraint"""
         self.assertEquals(self.table.c.data.nullable, True)
 
-        # Column object
-        self.table.c.data.alter(Column('data', String(40), nullable=False))
+        # Full column
+        self.table.c.data.alter(type=String(40), nullable=False)
         self.table.nullable = None
         self.refresh_table(self.table.name)
         self.assertEquals(self.table.c.data.nullable, False)
@@ -544,24 +685,30 @@ class TestColumnChange(fixture.DB):
         self.assertEquals(self.table.c.data.nullable, True)
 
     @fixture.usedb()
-    def test_alter_metadata(self):
-        """Test if alter_metadata is respected"""
-
-        self.table.c.data.alter(Column('data', String(100)))
-
-        self.assert_(isinstance(self.table.c.data.type, String))
-        self.assertEqual(self.table.c.data.type.length, 100)
-
-        # nothing should change
-        self.table.c.data.alter(Column('data', String(200)), alter_metadata=False)
-        self.assert_(isinstance(self.table.c.data.type, String))
-        self.assertEqual(self.table.c.data.type.length, 100)
+    def test_alter_deprecated(self):
+        try:
+            # py 2.4 compatability :-/
+            cw = catch_warnings(record=True)
+            w = cw.__enter__()
+            
+            warnings.simplefilter("always")
+            self.table.c.data.alter(Column('data', String(100)))
 
+            self.assertEqual(len(w),1)
+            self.assertTrue(issubclass(w[-1].category,
+                                       MigrateDeprecationWarning))
+            self.assertEqual(
+                'Passing a Column object to alter_column is deprecated. '
+                'Just pass in keyword parameters instead.',
+                str(w[-1].message))
+        finally:
+            cw.__exit__()
+            
     @fixture.usedb()
     def test_alter_returns_delta(self):
         """Test if alter constructs return delta"""
 
-        delta = self.table.c.data.alter(Column('data', String(100)))
+        delta = self.table.c.data.alter(type=String(100))
         self.assert_('type' in delta)
 
     @fixture.usedb()
@@ -580,8 +727,7 @@ class TestColumnChange(fixture.DB):
         kw = dict(nullable=False,
                  server_default='foobar',
                  name='data_new',
-                 type=String(50),
-                 alter_metadata=True)
+                 type=String(50))
         if self.engine.name == 'firebird':
             del kw['nullable']
         self.table.c.data.alter(**kw)
@@ -678,23 +824,17 @@ class TestColumnDelta(fixture.DB):
         self.verify([], self.mkcol(server_default='foobar'), self.mkcol('id', String, DefaultClause('foobar')))
         self.verify(['type'], self.mkcol(server_default='foobar'), self.mkcol('id', Text, DefaultClause('foobar')))
 
-        # test alter_metadata
         col = self.mkcol(server_default='foobar')
         self.verify(['type'], col, self.mkcol('id', Text, DefaultClause('foobar')), alter_metadata=True)
         self.assert_(isinstance(col.type, Text))
 
         col = self.mkcol()
-        self.verify(['name', 'server_default', 'type'], col, self.mkcol('beep', Text, DefaultClause('foobar')), alter_metadata=True)
+        self.verify(['name', 'server_default', 'type'], col, self.mkcol('beep', Text, DefaultClause('foobar')),
+                    alter_metadata=True)
         self.assert_(isinstance(col.type, Text))
         self.assertEqual(col.name, 'beep')
         self.assertEqual(col.server_default.arg, 'foobar')
 
-        col = self.mkcol()
-        self.verify(['name', 'server_default', 'type'], col, self.mkcol('beep', Text, DefaultClause('foobar')), alter_metadata=False)
-        self.assertFalse(isinstance(col.type, Text))
-        self.assertNotEqual(col.name, 'beep')
-        self.assertFalse(col.server_default)
-
     @fixture.usedb()
     def test_deltas_zero_columns(self):
         """Testing ColumnDelta with zero columns"""
@@ -705,24 +845,18 @@ class TestColumnDelta(fixture.DB):
         self.verify(['type'], 'ids', table=self.table.name, type=String(80), engine=self.engine)
         self.verify(['type'], 'ids', table=self.table.name, type=String(80), metadata=self.meta)
 
-        # check if alter_metadata is respected
         self.meta.clear()
-        delta = self.verify(['type'], 'ids', table=self.table.name, type=String(80), alter_metadata=True, metadata=self.meta)
+        delta = self.verify(['type'], 'ids', table=self.table.name, type=String(80), metadata=self.meta,
+                            alter_metadata=True)
         self.assert_(self.table.name in self.meta)
         self.assertEqual(delta.result_column.type.length, 80)
         self.assertEqual(self.meta.tables.get(self.table.name).c.ids.type.length, 80)
 
-        self.meta.clear()
-        self.verify(['type'], 'ids', table=self.table.name, type=String(80), alter_metadata=False, engine=self.engine)
-        self.assert_(self.table.name not in self.meta)
-
-        self.meta.clear()
-        self.verify(['type'], 'ids', table=self.table.name, type=String(80), alter_metadata=False, metadata=self.meta)
-        self.assert_(self.table.name not in self.meta)
-
         # test defaults
         self.meta.clear()
-        self.verify(['server_default'], 'ids', table=self.table.name, server_default='foobar', alter_metadata=True, metadata=self.meta)
+        self.verify(['server_default'], 'ids', table=self.table.name, server_default='foobar',
+                    metadata=self.meta,
+                    alter_metadata=True)
         self.meta.tables.get(self.table.name).c.ids.server_default.arg == 'foobar'
 
         # test missing parameters
@@ -746,17 +880,11 @@ class TestColumnDelta(fixture.DB):
         self.assertEquals(delta.get('name'), 'blah')
         self.assertEquals(delta.current_name, 'id')
 
-        # check if alter_metadata is respected
         col_orig = self.mkcol(primary_key=True)
         self.verify(['name', 'type'], col_orig, name='id12', type=Text, alter_metadata=True)
         self.assert_(isinstance(col_orig.type, Text))
         self.assertEqual(col_orig.name, 'id12')
 
-        col_orig = self.mkcol(primary_key=True)
-        self.verify(['name', 'type'], col_orig, name='id12', type=Text, alter_metadata=False)
-        self.assert_(isinstance(col_orig.type, String))
-        self.assertEqual(col_orig.name, 'id')
-
         # test server default
         col_orig = self.mkcol(primary_key=True)
         delta = self.verify(['server_default'], col_orig, DefaultClause('foobar'))
