@@ -10,7 +10,8 @@ from sqlalchemy.sql import and_
 from sqlalchemy import exceptions as sa_exceptions
 from sqlalchemy.sql import bindparam
 
-from migrate.versioning import exceptions, genmodel, schemadiff
+from migrate import exceptions
+from migrate.versioning import genmodel, schemadiff
 from migrate.versioning.repository import Repository
 from migrate.versioning.util import load_model
 from migrate.versioning.version import VerNum
@@ -22,7 +23,7 @@ class ControlledSchema(object):
     """A database under version control"""
 
     def __init__(self, engine, repository):
-        if isinstance(repository, str):
+        if isinstance(repository, basestring):
             repository = Repository(repository)
         self.engine = engine
         self.repository = repository
@@ -107,8 +108,9 @@ class ControlledSchema(object):
         model = load_model(model)
 
         diff = schemadiff.getDiffOfModelAgainstDatabase(
-            model, self.engine, excludeTables=[self.repository.version_table])
-        genmodel.ModelGenerator(diff).applyModel()
+            model, self.engine, excludeTables=[self.repository.version_table]
+            )
+        genmodel.ModelGenerator(diff,self.engine).applyModel()
 
         self.update_repository_table(self.version, int(self.repository.latest))
 
@@ -206,5 +208,6 @@ class ControlledSchema(object):
             repository = Repository(repository)
 
         diff = schemadiff.getDiffOfModelAgainstDatabase(
-            MetaData(), engine, excludeTables=[repository.version_table])
-        return genmodel.ModelGenerator(diff, declarative).toPython()
+            MetaData(), engine, excludeTables=[repository.version_table]
+            )
+        return genmodel.ModelGenerator(diff, engine, declarative).toPython()
